<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

class DLM_CE_Logs_Export {

	/**
	 * Constructor
	 *
	 * @param $data
	 */
	public function __construct() {

		add_action( 'admin_init', array( $this, 'catch_export_request' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'add_scripts' ) );
		add_action( 'wp_ajax_dlm_download_log', array( $this, 'run' ) );
		add_action( 'dlm_insights_header', array( $this, 'add_reports_export_button' ), 5 );
		add_action( 'wp_ajax_dlm_ce_update_export_settings', array( $this, 'update_export_settings' ) );

		add_action( 'wp_ajax_dlm_export_overview', array( $this, 'run_reports_overview' ) );
		// Add option for default export settings.
		add_option(
			'dlm_export_columns',
			array(
				'id',
				'title',
				'version_id',
				'file_name',
				'user_id',
				'user_login',
				'user_email',
				'user_ip',
				'download_date',
				'download_status',
				'download_meta'
			)
		);
		// Add the option for the overview export settings.
		add_option(
			'dlm_overview_export_columns',
			array(
				'id',
				'title',
				'total',
			)
		);
	}

	/**
	 * Add admin scripts
	 *
	 * @return void
	 * @since 4.0.4
	 */
	public function add_scripts() {
		$screen = get_current_screen();
		// Only enqueue the script on the Reports page.grunt ugli
		if ( 'dlm_download_page_download-monitor-reports' === $screen->base ) {
			wp_enqueue_script( 'dlm_csv_export_logs', DLM_CSVE_URL . 'assets/js/dlm-ce-logs-export' . ( ( ! SCRIPT_DEBUG ) ? '.min' : '' ) . '.js', array( 'jquery' ), DLM_CSV_Exporter::VERSION );
			wp_add_inline_script( 'dlm_csv_export_logs', 'const dlm_ce_nonce = "' . wp_create_nonce( 'dlm_ce_export' ) . '"', 'before' );
			wp_enqueue_style( 'dlm_csv_export_logs', DLM_CSVE_URL . 'assets/css/dlm_csv_export_admin' . ( ( ! SCRIPT_DEBUG ) ? '.min' : '' ) . '.css', array(), DLM_CSV_Exporter::VERSION );
		}
	}

	/**
	 * Add the export button to the Reports page.
	 *
	 * @return void
	 * @since 4.0.4
	 */
	public function add_reports_export_button() {

		$user_reports_checked = get_option( 'dlm_export_columns', array() );
		$overview_checked     = get_option( 'dlm_overview_export_columns', array() );
		?>
		<div class="dlm-csv-export-wrapper">
			<div class="dlm-reports-header-export-button">
				<a href="<?php echo esc_url( add_query_arg( 'dlm_download_logs', 'true', admin_url( 'edit.php' ) ) ); ?>"
				   target="_blank" id="dlm-download-log"
				   class="button button-primary"><?php echo esc_html__( 'Export', 'dlm-csv-exporter' ); ?></a>
			</div>
			<div class="dlm-csv-export-wrapper__export_settings">
				<a href="#" id="dlm-export-settings"
				   class="button button-secondary"><span class="dashicons dashicons-admin-generic"></span></a>
				<div id="user_reports_settings" class="dlm-export-settings__body">
					<?php
					$export_columns  = $this->get_user_reports_export_columns();

					if ( ! empty( $export_columns ) ) {
						echo '<ul>';
						foreach ( $export_columns as $column => $label ) {
							$checked = in_array( $column, $user_reports_checked ) ? 'checked = "checked"' : '';
							echo '<li><input type="checkbox" name="dlm_export_columns[]"  value="' . esc_attr( $column ) . '" ' . $checked . ' /> ' . esc_html( $label ) . '</li>';
						}
						echo '</ul>';
					}
					?>
				</div>
				<div id="general_info_settings" class="dlm-export-settings__body current">
					<?php
					$export_columns  = $this->get_overview_export_columns();

					if ( ! empty( $export_columns ) ) {
						echo '<ul>';
						foreach ( $export_columns as $column => $label ) {
							$checked = in_array( $column, $overview_checked ) ? 'checked = "checked"' : '';
							echo '<li><input type="checkbox" name="dlm_export_columns[]"  value="' . esc_attr( $column ) . '" ' . $checked . ' /> ' . esc_html( $label ) . '</li>';
						}
						echo '</ul>';
					}
					?>
				</div>
			</div>
		</div>
		<?php
	}

	/**
	 * Check if is allowed
	 *
	 * @return bool
	 */
	private function is_allowed() {

		if ( empty( $_GET['dlm_download_logs'] ) ) {
			return false;
		}

		if ( ! current_user_can( 'manage_downloads' ) ) {
			return false;
		}

		return true;
	}

	/**
	 * Catch the export request
	 */
	public function catch_export_request() {
		if ( isset( $_GET['dlm_download_logs'] ) ) {
			$this->run();
		}
	}

	/**
	 * Run
	 */
	public function run() {

		if ( ! isset( $_REQUEST['nonce'] ) ) {
			wp_die( "You're not allowed to export logs." );
		}

		wp_verify_nonce( $_REQUEST['nonce'], 'dlm_ce_export' ); // phpcs:ignore

		// check if allowed.
		if ( ! $this->is_allowed() ) {
			wp_die( "You're not allowed to export logs." );
		}

		$filter_user_id         = '';
		$filter_download_status = '';
		$filters                = array();

		if ( isset( $_REQUEST['user_filters'] ) && ! empty( $_REQUEST['user_filters'] ) ) {
			$user_filters = json_decode( sanitize_text_field( wp_unslash( $_REQUEST['user_filters'] ) ) );
			foreach ( $user_filters as $user_filter ) {

				switch ( $user_filter->type ) {
					case 'user_id':
						$filter_user_id = $user_filter->on;
						break;
					case 'download_status':
						$filter_download_status = $user_filter->on;
						break;
				}
			}
		}
		// Set new filters.
		if ( isset( $_GET['start_date'] ) ) {
			$filters[] = array(
				'key'      => 'download_date',
				'value'    => date( 'Y-m-d', strtotime( $_GET['start_date'] ) ),
				'operator' => '>=',
			);
		}

		if ( isset( $_GET['end_date'] ) ) {
			$filters[] = array(
				'key'      => 'download_date',
				'value'    => date( 'Y-m-d', strtotime( $_GET['end_date'] . ' +1 day' ) ),
				// Add 1 day to include the last day within the range
				'operator' => '<=',
			);
		}

		if ( ! empty( $filter_user_id ) ) {
			$filters[] = array(
				'key'      => 'user_id',
				'value'    => absint( $filter_user_id ),
				'operator' => '=',
			);
		}

		if ( ! empty( $filter_download_status ) ) {
			$filters[] = array(
				'key'      => 'download_status',
				'value'    => sanitize_text_field( $filter_download_status ),
				'operator' => '=',
			);
		}

		// get downloads
		$items = download_monitor()->service( 'log_item_repository' )->retrieve( apply_filters( 'dlm_csv_export_log_filters', $filters ) );

		// rows.
		$rows                   = array();
		$row                    = array();
		$default_export_columns = $this->get_user_reports_export_columns();
		$selected_columns       = get_option( 'dlm_export_columns', array() );
		$cr                     = $this->create_columns_row( $default_export_columns, $selected_columns );
		$rows[]                 = $cr['row'];
		$export_columns         = $cr['columns'];
		$rows                   = apply_filters( 'dlm_csv_export_log_rows', $rows );

		if ( ! empty( $items ) ) {

			/** @var DLM_Log_Item $item */
			foreach ( $items as $item ) {
				$row   = array();
				try {
					/** @var DLM_Download $download */
					$download = download_monitor()->service( 'download_repository' )->retrieve_single( $item->get_download_id() );
				} catch ( Exception $e ) {
					$download = new DLM_Download();
				}

				try {
					$version = download_monitor()->service( 'version_repository' )->retrieve_single( $item->get_version_id() );
					$download->set_version( $version );
				} catch ( Exception $e ) {

				}

				foreach ( $export_columns as $column => $label ) {
					switch ( $column ) {
						case 'id':
							// Adding ID to CSV.
							$row[] = $item->get_download_id();
							break;
						case 'title':
							// Adding Title to CSV.
							if ( $download->exists() ) {
								$row[] = esc_html( $download->get_title() );
							} else {
								$row[] = '-';
							}
							break;
						case 'version_id':
							// Adding version id to CSV.
							$row[] = $item->get_version_id();
							break;
						case 'file_name':
							// Adding file name to CSV.
							if ( $download->exists() && $download->get_version()->get_filename() ) {
								$row[] = $download->get_version()->get_filename();
							} else {
								$row[] = '-';
							}
							break;
						case 'user_id':
							// Adding User ID to CSV.
							$row[] = $item->get_user_id();

							if ( $item->get_user_id() ) {
								$user = get_user_by( 'id', $item->get_user_id() );
							}
							break;
						case 'user_email':
							// Adding user login & email to CSV.
							if ( ! isset( $user ) || ! $user ) {
								$row[] = '-';
							} else {
								$row[] = $user->user_email;
							}
							break;
						case 'user_login':
							// Adding user login & email to CSV.
							if ( ! isset( $user ) || ! $user ) {
								$row[] = '-';
							} else {
								$row[] = $user->user_login;
							}
							break;
						case 'user_ip':
							// Adding user IP to CSV.
							if ( ! empty( $item->get_user_ip() ) ) {
								$row[] = $item->get_user_ip();
							} else {
								$row[] = '-';
							}
							break;
						case 'download_date':
							// Adding download date to CSV.
							$row[] = $item->get_download_date()->format( 'Y-m-d H:i:s' );
							break;
						case 'download_status':
							// Adding status to CSV.
							$row[] = $item->get_download_status();
							break;
						case 'download_meta':
							// setup meta data string.
							$meta     = $item->get_meta_data();
							$meta_str = '';
							if ( ! empty( $meta ) ) {
								foreach ( $meta as $mk => $mv ) {
									$meta_str .= $mk . ': ' . $mv . '\n';
								}
							}
							// Adding meta data to CSV.
							$row[] = $meta_str;
							break;
						default:
							// Adding custom columns to CSV.
							$row[] = apply_filters( 'dlm_csv_export_log_row_column_' . $column, '', $column, $item );
							break;
					}
				}

				unset( $user );

				$row = apply_filters( 'dlm_csv_export_log_rows', $row, $item, $download );

				$rows[] = '"' . implode( '","', $row ) . '"';
			}
		}

		$log = implode( "\n", $rows );

		header( 'Content-type: text/csv' );
		header( 'Content-Disposition: attachment; filename=download_log.csv' );
		header( 'Cache-Control: must-revalidate, post-check=0, pre-check=0' );
		header( 'Content-Length: ' . strlen( $log ) );
		echo wp_kses_post( $log );
		exit;
	}

	/**
	 * Save export settings
	 *
	 * @return void
	 * @since 4.0.5
	 */
	public function update_export_settings() {
		// Check nonce.
		check_ajax_referer( 'dlm_ce_export', 'nonce' );
		// Verify that the current user has permission to update options.
		if ( ! isset( $_POST['action'] ) || 'dlm_ce_update_export_settings' !== $_POST['action'] || ! current_user_can( 'manage_options' ) ) {
			wp_send_json_error();
		}
		// Check for values.
		if ( isset( $_POST['values'] ) && ! empty( $_POST['values'] ) ) {
			if ( isset( $_POST['setting'] ) ) {
				if ( 'user_reports_settings' === $_POST['setting'] ) {
					update_option( 'dlm_export_columns', array_map( 'sanitize_text_field', wp_unslash( $_POST['values'] ) ) );
				} elseif ( 'general_info_settings' === $_POST['setting'] ) {
					update_option( 'dlm_overview_export_columns', array_map( 'sanitize_text_field', wp_unslash( $_POST['values'] ) ) );
				} else {
					do_action( 'dlm_custom_export_' . sanitize_option( $_POST['setting'] ), $_POST );
				}
			}
		} else {
			wp_send_json_error( __( 'No columns selected', 'dlm-csv-exporter' ) );
		}
		// All good, send success.
		wp_send_json_success();
	}

	/**
	 * The default export columns
	 *
	 * @return mixed|null
	 * @since 4.0.5
	 */
	public function get_user_reports_export_columns() {
		return apply_filters(
			'dlm_export_columns',
			array(
				'id'              => __( 'ID', 'dlm-csv-exporter' ),
				'title'           => __( 'Title', 'dlm-csv-exporter' ),
				'version_id'      => __( 'Version ID', 'dlm-csv-exporter' ),
				'file_name'       => __( 'File name', 'dlm-csv-exporter' ),
				'user_id'         => __( 'User ID', 'dlm-csv-exporter' ),
				'user_login'      => __( 'User login', 'dlm-csv-exporter' ),
				'user_email'      => __( 'User email', 'dlm-csv-exporter' ),
				'user_ip'         => __( 'User IP', 'dlm-csv-exporter' ),
				'download_date'   => __( 'Download date', 'dlm-csv-exporter' ),
				'download_status' => __( 'Download status', 'dlm-csv-exporter' ),
				'download_meta'   => __( 'Download meta', 'dlm-csv-exporter' ),
			)
		);
	}

	/**
	 * The default export columns
	 *
	 * @return mixed|null
	 * @since 4.0.9
	 */
	public function get_overview_export_columns() {

		$defaults = array(
			'id'    => __( 'ID', 'dlm-csv-exporter' ),
			'title' => __( 'Title', 'dlm-csv-exporter' ),
			'total' => __( 'Total', 'dlm-csv-exporter' ),
		);

		return array_merge(
			$defaults,
			apply_filters(
				'dlm_overview_export_columns',
				array()
			)
		);
	}

	/**
	 * The default export columns
	 *
	 * @since 4.0.9
	 */
	public function run_reports_overview() {
		check_ajax_referer( 'dlm_ce_export', 'nonce' );

		$request_body = file_get_contents( 'php://input' );
		$data         = json_decode( $request_body, true );

		if ( ! isset( $data['downloads'] ) ) {
			echo 'Nope!';
			exit;
		}

		$downloads        = $data['downloads'];
		$export_columns   = $this->get_overview_export_columns();
		$selected_columns = get_option( 'dlm_overview_export_columns', array() );
		$cr               = $this->create_columns_row( $export_columns, $selected_columns );
		$columns          = $cr['columns'];
		$rows[]           = $cr['row'];
		$rows             = apply_filters( 'dlm_csv_overview_export_log_rows', $rows );

		if ( ! empty( $downloads ) ) {

			/** @var DLM_Log_Item $item */
			foreach ( $downloads as $download ) {
				$row = array();
				foreach ( $columns as $column => $label ) {
					if ( isset( $download[ $column ] ) ) {
						$row[] = $download[ $column ];
						continue;
					}
					$row[] = 0;
				}

				$row = apply_filters( 'dlm_csv_overview_export_log_row', $row, $download );

				$rows[] = '"' . implode( '","', $row ) . '"';
			}
		}

		$log = implode( "\n", $rows );

		header( 'Content-type: text/csv' );
		header( 'Content-Disposition: attachment; filename=download_overview_log.csv' );
		header( 'Cache-Control: must-revalidate, post-check=0, pre-check=0' );
		header( 'Content-Length: ' . strlen( $log ) );
		echo wp_kses_post( $log );
		exit;
	}

	/**
	 * Create table headers
	 *
	 * @param array $export_columns   Available export columns.
	 * @param array $selected_columns Selected export columns.
	 *
	 * @return array
	 * @since 4.0.9
	 */
	private function create_columns_row( $export_columns, $selected_columns ) {
		$row     = array();
		$columns = array();
		// Let's check if we have selected columns.
		foreach ( $export_columns as $column_key => $column_name ) {
			if ( in_array( $column_key, $selected_columns ) ) {
				$columns[ $column_key ] = $column_name;
			}
		}

		// Let's set the selected columns headers.
		if ( ! empty( $columns ) ) {
			foreach ( $columns as $column => $label ) {
				$row[] = $label;
			}
		}

		return array(
			'columns' => $columns,
			'row'     => '"' . implode( '","', $row ) . '"'
		);
	}
}
